// ニジーク・リモコン  (c)2014 Ryosuke Kawano

#include <avr/sleep.h>

// 定数定義
const int SW_FWD = 2;
const int SW_REV = 3;
const int SW_LEFT = 7;
const int SW_RIGHT = 8;
const int ID_PIN = 9;
const int EXT_INT = 0;       // exrernal int 0 : D2
const int LED_IR = 11;
const int LED_RED = 13;
const int ON = 1;
const int OFF = 0;
const int SLEEP_TIME = ( 10000 / 120 );

// 変数定義
int fwd;
int rev;
int left;
int right;
int id;
int parity;
int sleep_counter;

void setup( ){
  // 初期化状態
  pinMode( SW_FWD, INPUT_PULLUP );
  pinMode( SW_REV, INPUT_PULLUP );
  pinMode( SW_LEFT, INPUT_PULLUP );
  pinMode( SW_RIGHT, INPUT_PULLUP );
  pinMode( ID_PIN, INPUT_PULLUP );
  pinMode( LED_RED, OUTPUT );
  pinMode( LED_IR, OUTPUT );
  bitClear( ADCSRA, ADEN );  // A/D変換禁止
  bitSet( PRR, PRADC );      // A/D変換器電力削減
  set_sleep_mode( SLEEP_MODE_PWR_DOWN );
  // 初期出力
  digitalWrite( LED_RED, HIGH );
  sleep_counter = 0;
}

void loop( ){
  input_switches( );
  if ( id == 0 ){      // ID = 0 の送信処理
    transmit_data( );
    delay( 48 );
    transmit_data( );
    delay( 49 );
  } else {             // ID = 1 の送信処理
    transmit_data( );
    delay( 18 );
    transmit_data( );
    delay( 79 );
  }
  sleep_control( );
}

void input_switches( void ){
  fwd = !digitalRead( SW_FWD );
  rev = !digitalRead( SW_REV );
  left = !digitalRead( SW_LEFT );
  right = !digitalRead( SW_RIGHT );
  if ( ( fwd == ON )
    && ( rev == ON )){
    fwd = OFF;
    rev = OFF;
  }
  if ( ( left == ON )
    && ( right == ON )){
    left = OFF;
    right = OFF;
  }
  id = !digitalRead( ID_PIN );
}

void transmit_data( void ){
  parity = 1;
  tone( LED_IR, 38000 );
  delayMicroseconds( 1385 );  // 3000μs
  noTone( LED_IR );
  delayMicroseconds( 525 );   // 600μs
  transmit_bit( id );
  transmit_bit( fwd );
  transmit_bit( rev );
  transmit_bit( left );
  transmit_bit( right );
  transmit_bit( parity );
}

void transmit_bit( int bitdata ){
  tone( LED_IR, 38000 );
  if ( bitdata == 0 ){
    delayMicroseconds( 275 ); // 600μs
  } else {
    delayMicroseconds( 551 ); // 1200μs
    parity = !parity;
  }
  noTone( LED_IR );
  delayMicroseconds( 525 );   // 600μs
}

void sleep_control( void ){
  sleep_counter++;
  if ( ( fwd == ON )
    || ( rev == ON )
    || ( left == ON )
    || ( right == ON ) ){
    sleep_counter = 0;
  }
  if ( sleep_counter >= SLEEP_TIME ){
    // スリープする
    digitalWrite( LED_RED, LOW );
    pinMode( ID_PIN, OUTPUT );
    digitalWrite( ID_PIN, LOW );
    attachInterrupt( EXT_INT, ir_wakeup, FALLING );
    sleep_mode( );
    // SW_FWD プッシュスイッチが押された
    // ウェイクアップする
    detachInterrupt( EXT_INT );
    pinMode( ID_PIN, INPUT_PULLUP );
    digitalWrite( LED_RED, HIGH );
    sleep_counter = 0;
  }
}

void ir_wakeup( ){
  ;
}

