// 遊歩ロボット・ニジーク test13　(c)2014 Ryosuke Kawano

#include <Servo.h>

Servo  myservo;

const int IR_RECEIVE = 4;
const int ID_PIN = 3;
const int FWD_PIN = 7;
const int REV_PIN = 8;
const int SRV_PIN = 9;
const int TEST_PIN = 10;

const char IDLE_STATE = 0;
const char HEADER_STATE = 1;
const char GAP_STATE = 2;
const char ID_STATE = 3;
const char POST_ID_STATE = 4;
const char FWD_STATE = 5;
const char POST_FWD_STATE = 6;
const char REV_STATE = 7;
const char POST_REV_STATE = 8;
const char LEFT_STATE = 9;
const char POST_LEFT_STATE = 10;
const char RIGHT_STATE = 11;
const char POST_RIGHT_STATE = 12;
const char PARITY_STATE = 13;
const char ON = 1;
const char OFF = 0;
const int CENTER = 90;
const int LEFT = ( CENTER + 50 );
const int RIGHT = ( CENTER - 50 );
const char TICK_TIME = 100;     // 刻み時間 μs
const char HEADER_DETECT_TIME = ( 1800 / TICK_TIME );
const char SHORT_BIT = ( 600 / TICK_TIME );
const char LONG_BIT = ( 1200 / TICK_TIME );
const char JUDGE_BIT = ( ( SHORT_BIT + LONG_BIT ) / 2 );

char input_level = HIGH;
char receive_state = IDLE_STATE;
char header_counter = 0;
char header_detect = OFF;
char bit_receive_counter = 0;
char id_receive = 0;
char fwd_receive = OFF;
char rev_receive = OFF;
char left_receive = OFF;
char right_receive = OFF;
char parity_temp = 0;
char fwd = OFF;
char rev = OFF;
char left = OFF;
char right = OFF;

void setup( ){
  pinMode( IR_RECEIVE, INPUT_PULLUP );
  pinMode( ID_PIN, INPUT_PULLUP );
  pinMode( FWD_PIN, OUTPUT );
  pinMode( REV_PIN, OUTPUT );
  pinMode( SRV_PIN, OUTPUT );
  pinMode( TEST_PIN, OUTPUT );
  myservo.attach( SRV_PIN );
  myservo.write( CENTER );
}

void loop( ){
  digitalWrite( TEST_PIN, HIGH );
  input_level = digitalRead( IR_RECEIVE );
  header_check( );
  receive_data( );

  move_and_swing( );
  
  digitalWrite( TEST_PIN, LOW );
  delayMicroseconds( 10 );
}

void header_check( void ){
  header_counter++;
  if ( input_level == HIGH ){
    header_counter = 0;
    header_detect = OFF;
  }
  if ( header_counter >= HEADER_DETECT_TIME ){
    header_counter = HEADER_DETECT_TIME;
    header_detect = ON;
  }
}

void receive_data( void ){
  switch ( receive_state ){
  case IDLE_STATE:
    if ( header_detect == ON ){
      receive_state = HEADER_STATE;
    }
    break;
  case HEADER_STATE:
    if ( input_level == HIGH ){
      bit_receive_counter = 0;
      receive_state = GAP_STATE;
    }
    break;
  case GAP_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( SHORT_BIT + 2 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == LOW ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          parity_temp = 0;
          bit_receive_counter = 0;
          receive_state = ID_STATE;
        }
      }
    }
    break;
  case ID_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( LONG_BIT + 3 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == HIGH ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          id_receive = 0;
          if ( bit_receive_counter >= JUDGE_BIT ){
            id_receive = 1;
            parity_temp = !parity_temp;
          }
          if ( id_receive == !digitalRead( ID_PIN ) ){
            bit_receive_counter = 0;
            receive_state = POST_ID_STATE;
          } else {
            receive_state = IDLE_STATE;
          }
        }
      }
    }
    break;
  case POST_ID_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( SHORT_BIT + 2 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == LOW ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          bit_receive_counter = 0;
          receive_state = FWD_STATE;
        }
      }
    }
    break;
  case FWD_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( LONG_BIT + 3 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == HIGH ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          fwd_receive = 0;
          if ( bit_receive_counter >= JUDGE_BIT ){
            fwd_receive = 1;
            parity_temp = !parity_temp;
          }
          bit_receive_counter = 0;
          receive_state = POST_FWD_STATE;
        }
      }
    }
    break;
  case POST_FWD_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( SHORT_BIT + 2 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == LOW ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          bit_receive_counter = 0;
          receive_state = REV_STATE;
        }
      }
    }
    break;
  case REV_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( LONG_BIT + 3 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == HIGH ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          rev_receive = 0;
          if ( bit_receive_counter >= JUDGE_BIT ){
            rev_receive = 1;
            parity_temp = !parity_temp;
          }
          bit_receive_counter = 0;
          receive_state = POST_REV_STATE;
        }
      }
    }
    break;
  case POST_REV_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( SHORT_BIT + 2 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == LOW ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          bit_receive_counter = 0;
          receive_state = LEFT_STATE;
        }
      }
    }
    break;
  case LEFT_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( LONG_BIT + 3 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == HIGH ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          left_receive = 0;
          if ( bit_receive_counter >= JUDGE_BIT ){
            left_receive = 1;
            parity_temp = !parity_temp;
          }
          bit_receive_counter = 0;
          receive_state = POST_LEFT_STATE;
        }
      }
    }
    break;
  case POST_LEFT_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( SHORT_BIT + 2 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == LOW ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          bit_receive_counter = 0;
          receive_state = RIGHT_STATE;
        }
      }
    }
    break;
  case RIGHT_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( LONG_BIT + 3 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == HIGH ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          right_receive = 0;
          if ( bit_receive_counter >= JUDGE_BIT ){
            right_receive = 1;
            parity_temp = !parity_temp;
          }
          bit_receive_counter = 0;
          receive_state = POST_RIGHT_STATE;
        }
      }
    }
    break;
  case POST_RIGHT_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( SHORT_BIT + 2 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == LOW ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          bit_receive_counter = 0;
          receive_state = PARITY_STATE;
        }
      }
    }
    break;
  case PARITY_STATE:
    bit_receive_counter++;
    if ( bit_receive_counter > ( LONG_BIT + 3 ) ){
      receive_state = IDLE_STATE;
    } else {
      if ( input_level == HIGH ){
        if ( bit_receive_counter < ( SHORT_BIT - 2 ) ){
          receive_state = IDLE_STATE;
        } else {
          if ( bit_receive_counter >= JUDGE_BIT ){
            parity_temp = !parity_temp;
          }
          if ( parity_temp == 1 ){
            fwd = fwd_receive;
            rev = rev_receive;
            left = left_receive;
            right = right_receive;
          }
          receive_state = IDLE_STATE;
        }
      }
    }
    break;
  }
}

void move_and_swing( void ){
  if ( fwd == ON ){
    digitalWrite( FWD_PIN, HIGH );
  } else {
    digitalWrite( FWD_PIN, LOW );
  }
  if ( rev == ON ){
    digitalWrite( REV_PIN, HIGH );
  } else {
    digitalWrite( REV_PIN, LOW );
  }
  if ( left == ON ){
    myservo.write( LEFT );
  } else if ( right == ON ){
    myservo.write( RIGHT );
  } else {
    myservo.write( CENTER );
  }
}

